function [result, result2, av_data, auto_av_signal] = processdata2001(data,no_frames);
%  Generates the autocorrelation of the data, ignoring the effects of noise
%  and background effects for the number of frames specified. 
%
%  function [raw, ac, av, norm] = processdata2001(data, no_frames); 
%
%  Inputs:	    data			- filename of the scintillation data
% 				no_frames	    - the number of frames of each used to calculate the
%								  autocorrelation.
%
%  Outputs:	    raw		        - unnormalised average scintillation covariance
%				result2		    - normalised average scintillation covariance   
%  			    av		        - long exposure (average) scintillation data
%               norm            - scintillation covariance normalisation term
%
%  Usage:  [raw, ac, av, norm] = processdata2001('Run15.dat', 2000);
%
%  Rachel Johnston, 2001
% Updated with wait bar for user interface.
s = which (data);
if (isempty(s))
   disp ('Requested Data File is not existing in Scidar_data directory')
   return   
   end

fid_data = fopen(data, 'r');
header = fread(fid_data, 32768, 'uchar');
cleanup = fread(fid_data, 32768, 'uchar');
cleanup = fread(fid_data, 32768, 'uchar');

% Account for any cleanup frames and short fall of frames
if (no_frames >= 1000)
    no_frames = no_frames - 10;
end

data_sum = zeros(128);
auto_data_sum = zeros(256);

h = waitbar(0,'Processing. Please wait...');
for i = 1:no_frames
   data = fread(fid_data, [128, 128], 'uchar');
   data_sum = data_sum + data;
	data_pack = zeros(256);
	data_pack(1:128,1:128) = data;
	ft_data = fft2(data_pack);
	auto_data = ifft2(ft_data.*conj(ft_data));
	auto_data_sum = auto_data_sum + auto_data;
    waitbar(i/no_frames)

end

fclose(fid_data);
if (rem(no_frames,100)~=0)
    disp('100 percent of frames read');
 end
close(h)     


av_data = data_sum/no_frames;                   % Average (Long Exposure) Scintillation Image
av_auto_data = auto_data_sum/no_frames;

av_data_pack = zeros(256);
av_data_pack(1:128,1:128) = av_data;	
ft_av_data = fft2(av_data_pack);
auto_av_data = ifft2(ft_av_data.*conj(ft_av_data));

av_auto_signal = av_auto_data - auto_av_data;
av_signal = av_data;
av_signal_pack = zeros(256);
av_signal_pack(1:128,1:128) = av_signal;
ft_av_signal = fft2(av_signal_pack);
auto_av_signal = real(fftshift(ifft2(ft_av_signal.*conj(ft_av_signal))));

% Raw result - no normalisation
result = (real(fftshift(av_auto_signal)));

% Normalised result - central section extracted
unmaskresult = real(fftshift(av_auto_signal)./(auto_av_signal+eps));
unmaskresult_section = unmaskresult(129-63:129+64,129-63:129+64);
m = mean(mean(unmaskresult_section));
result2 = m*ones(256);
result2(129-63:129+64,129-63:129+64)= unmaskresult_section;
