function [mask] = cgmask(gi, f, hi, gm, fm, hm, reg_matrix);
% Calculates a new mask for f that excludes all pixels that are negative
% or likely to go negative in the next iteration using the following
% criterion.
% (f > 0) OR (Gradient of f < 0)
% i.e. for finding the active/inactive pixels when enforcing positivity
% Usage: function [mask] = cgmask(gi, f, hi, gm, fm, hm, reg_matrix);
% Rachel Johnston, 31/8/98

% First ensure that everything is the right size
f = f.*fm;

no_g = size(gi,3);
for n = 1:no_g
	g(:,:,n) = gi(:,:,n).*gm;
	h(:,:,n) = hi(:,:,n).*hm;
end

%Now find the autocorrelation of the hi
sizeg = size(gi,1);
AHsum = zeros(sizeg);
HGsum = zeros(sizeg);
for k = 1:no_g
	H = fft2(h(:,:,k));
	[rowh, colh] = size(h(:,:,k));
	hr = rot90(h(:,:,k),2);
	hs = zeros(size(hr));
	hs(2:rowh, 2:colh) = hr(1:rowh-1, 1:colh - 1);
	hs(1,2:colh) = hr(rowh, 1:colh-1);
	hs(2:rowh, 1) = hr(1:rowh - 1, colh);
	hs(1,1) = hr(rowh, colh);
	HC = fft2(hs);
	AH(:,:,k) = H.*HC;
	AHsum = AHsum + AH(:,:,k);
	HG(:,:,k) = HC.*fft2(g(:,:,k));
	HGsum = HGsum + HG(:,:,k);
end

%  Impose positivity constraint
%
f = f.* (f>0);
gradsum = cggrad(f, AHsum, HGsum, reg_matrix, fm);

mask = ((f > 0) | (gradsum < 0)) .* fm; 

