function [fest, error] = cgrad(f, hi, gi, fm, hm, gm, reg_matrix, error_matrix, no_iterations);
% Conjugate Gradient algorithm.  Generates  and estimate for f, given an initial 
% estimate for f as well as the hi and di.  It is minimizing the following
% cost function where it is assumed that an energy constraint is imposed on 
% f and h.
% E = sum2((conv2(fest, hi) - di).^2) + sum2(reg_matrix.*(fest.^2)) + sum2(reg_matrix.*(hest.^2))
% Usage: [fest, error] = cgrad(f, hi, gi, fm, hm, gm, reg_matrix, error_matrix, no_iterations);
% Rachel Johnston, 26/8/98

%First ensure that everything is the right size
f = f.*fm;

no_g = size(gi,3);
for n = 1:no_g
	g(:,:,n) = gi(:,:,n).*gm;
	h(:,:,n) = hi(:,:,n).*hm;
end

%Now find the autocorrelation of the hi
sizeg = size(gi,1);
AHsum = zeros(sizeg);
HGsum = zeros(sizeg);
for k = 1:no_g
	H = fft2(h(:,:,k));
	[rowh, colh] = size(h(:,:,k));
	hr = rot90(h(:,:,k),2);
	hs = zeros(size(hr));
	hs(2:rowh, 2:colh) = hr(1:rowh-1, 1:colh - 1);
	hs(1,2:colh) = hr(rowh, 1:colh-1);
	hs(2:rowh, 1) = hr(1:rowh - 1, colh);
	hs(1,1) = hr(rowh, colh);
	HC = fft2(hs);
	AH(:,:,k) = H.*HC;
	AHsum = AHsum + AH(:,:,k);
	HG(:,:,k) = HC.*fft2(g(:,:,k));
	HGsum = HGsum + HG(:,:,k);
end


gradsum = cggrad(f, AHsum, HGsum, reg_matrix, fm);
d = -gradsum;

fest = f;

for i = 1:no_iterations
	Qdsum = (real(ifft2(AHsum.*fft2(d))) + (reg_matrix.*d)).*fm;
	factor = sum(sum(d.*Qdsum));
   alpha = -sum(sum(gradsum.*d))/factor;
   fest = (fest + alpha*d).*fm;
	
	%alpha
	
	%Q = real(ifft2(AHsum));
	%Q
	%norm(Q(1:4,1:4))
	%sqrt(sum2(Q(1:4,1:4).^2))
	
	%norm(Q)
	
	%sqrt(sum2(Q.^2))

	%if alpha > 2/sqrt(sum2(Q.^2))
	%	disp('greater')
	%end	
	%t = Q.*f;
	%t(1:4,1:4)
	%pause
	
	%Calculating the new gradient
	gradsum = gradsum + alpha*Qdsum;
	beta = sum(sum(gradsum.*Qdsum))/factor;
	d = -gradsum + beta*d;
	
	%Calculating the error
	error(i) = cgerror(fest, hi, gi, gm, reg_matrix, error_matrix);

end
