function ok = fitswrite(header,image,name)
  
% FITSWRITE - function to write fits files.
%
%
%  Version 1.0 Beta 99-09-25 Peter Rydester, GNU General Public License.
%
%
%
% function ok = fitswrite(header,image,name)
%
%
%   header    A char matrix holding all fits file header
%             "keywords". All chars after possition 80
%             are truncated. Header char matrix is easy
%             created with a help off: fitsaddkey.
%             If you don't have any keywords to put in to
%             your header, then just put in '' as argument.
%
%             Image will be stored in datatype specified by
%             BITPIX keyword. Remember to set that keyword
%             to wanted datatype.
%
%   image     Image data matrix. Header keywords will be
%             modified to image matrix size. If
%
%   name      Path, name and extention for file to store in.
%
%       Original version Bjorn Gustavsson 98-5-25 wfits.m
% Modified         Peter Rydesaeter 98-5-26 fitswrite version
  
  ok=0;
  bitpix=fitsfindkeyvalue(header,'BITPIX');
  if isempty(bitpix),
  %  if isdouble(image),
  %    bitpix=-32;
  %  elseif isint16
      bitpix=-34;
  %  end
  end
  datatypesize=abs(bitpix)/8;
  image=double(image);
  
  si=size(image);
  
  % Remove old keywords
  header=fitsremovekey(header,'SIMPLE');
  header=fitsremovekey(header,'BITPIX');
  header=fitsremovekey(header,'NAXIS');
  header=fitsremovekey(header,'NAXIS1');
  header=fitsremovekey(header,'NAXIS2');
  header=fitsremovekey(header,'END');
  
  %Add new keywords.
  header=strvcat('SIMPLE  =                    T',header);
  header=fitsaddkey(header,'BITPIX',bitpix,2);
  header=fitsaddkey(header,'NAXIS' ,2,     3);
  header=fitsaddkey(header,'NAXIS1',si(2), 4);
  header=fitsaddkey(header,'NAXIS2',si(1), 5);
  
  header(end+1,1:80)=['END',blanks(77)];  % Add end line.
  header=header(:,1:80);     % Cut width of text matrix to 80.
  disp (header)
  disp ('I am displaying the fits header');
  
  %Open file for writing in rigth byte order in all platforms.
  %fid = fopen(name,'wb','ieee-be');
  fid = fopen(name,'w','n');
  if (fid == -1),
    msgstr = sprintf('Unable to open the file: %s\n',name);
    error(msgstr)
  end
  
  % Write header
  fprintf(fid,'file %s\n',name);
  fprintf(fid,'header %s\n',header');
  
  % Find number of bytes left to end of this 2880 block.
  i=2880-mod(ftell(fid),2880);
  while i>0,
  %  fwrite(fid,' ','char');
    i=i-1;
  end
  
  % Write data.
 
  switch bitpix,
  case 8,   fwrite(fid,image','uchar');
  case 16,  fwrite(fid,image','int16');
  case 32,  fwrite(fid,image','int32');
  case -32, fwrite(fid,image','float32');
  case -64, fwrite(fid,image','float64');
  end
  fclose(fid);
  ok = 1;
